<?php

declare(strict_types=1);

namespace Drupal\Tests\views_access_conditions\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\views_access_conditions\Traits\TestConditionTrait;
use Drupal\user\Entity\User;
use Drupal\views\Entity\View;
use Drupal\views\Tests\ViewTestData;

/**
 * Tests the views access conditions functionality.
 *
 * @group views_access_conditions
 */
class ViewsAccessConditionsTest extends KernelTestBase {

  use TestConditionTrait;

  /**
   * The modules to enable for this test.
   *
   * @var array
   */
  protected static $modules = [
    'system',
    'user',
    'field',
    'text',
    'node',
    'path_alias',
    'views',
    'views_access_conditions',
    'views_access_conditions_test',
  ];

  /**
   * Views used by this test.
   *
   * @var array
   */
  public static $testViews = ['test_access_conditions'];

  /**
   * The test user.
   *
   * @var \Drupal\user\Entity\User
   */
  protected ?User $user;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('user');
    $this->installEntitySchema('node');
    $this->installEntitySchema('path_alias');
    $this->installConfig(['system', 'user', 'field', 'text', 'node', 'path_alias', 'views']);

    // Create test views.
    ViewTestData::createTestViews(self::class, ['views_access_conditions_test']);

    // Create a test user.
    $this->user = User::create([
      'name' => 'test_user',
      'mail' => 'test@example.com',
      'status' => 1,
    ]);
    $this->user->save();

    // Set the current user.
    $this->container->get('current_user')->setAccount($this->user);
  }

  /**
   * Tests that conditions are properly added to the view access form.
   */
  public function testViewAccessFormAlter(): void {
    $view = View::load('test_access_conditions');
    // Configure the view with our test condition.
    $display_options = $view->getDisplay('default');
    $display_options['display_options']['access'] = [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
        ],
      ],
    ];
    $view->save();

    // Verify the conditions were saved correctly.
    $saved_view = View::load('test_access_conditions');
    $saved_display = $saved_view->getDisplay('default');
    $this->assertEquals('views_access_conditions', $saved_display['display_options']['access']['type']);
    $this->assertArrayHasKey('test_condition', $saved_display['display_options']['access']['options']['conditions']);
  }

  /**
   * Tests that conditions properly control view access.
   */
  public function testViewAccessConditions(): void {
    $view = View::load('test_access_conditions');
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');

    // Configure the view with our test condition.
    $display->setOption('access', [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
        ],
      ],
    ]);
    $view->save();

    /** @var \Drupal\views\Plugin\views\access\AccessPluginBase $access_plugin */
    $access_plugin = $display->getPlugin('access');

    // Assert access is allowed when condition is met.
    $this->assertTrue($access_plugin->access($this->user));

    $view = View::load('test_access_conditions');
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');

    // Change the condition to not be met.
    $display->setOption('access', [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => FALSE,
            'negate' => FALSE,
          ],
        ],
      ],
    ]);
    $view->save();

    $view = View::load('test_access_conditions');
    $display_options = $view->getDisplay('default');
    $this->assertFalse($display_options['display_options']['access']['options']['conditions']['test_condition']['condition_met']);

    // Reset the view executable to ensure we get a fresh instance.
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');

    /** @var \Drupal\views\Plugin\views\access\AccessPluginBase $access_plugin */
    $access_plugin = $display->getPlugin('access');

    // Assert access is forbidden when condition is not met.
    $this->assertFalse($access_plugin->access($this->user));
  }

  /**
   * Tests that multiple conditions work together properly.
   */
  public function testMultipleConditions(): void {
    $view = View::load('test_access_conditions');
    // Configure the view with two test conditions.
    $display_options = $view->getDisplay('default');
    $display_options['display_options']['access'] = [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
          'test_condition_two' => [
            'id' => 'test_condition_two',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
        ],
      ],
    ];
    $view->save();

    // Get the view executable.
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');

    /** @var \Drupal\views\Plugin\views\access\AccessPluginBase $access_plugin */
    $access_plugin = $display->getPlugin('access');

    // Assert access is allowed when both conditions are met.
    $this->assertTrue($access_plugin->access($this->user));

    // Change one condition to not be met.
    $display_options = $view->getDisplay('default');
    $display_options['display_options']['access'] = [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
          'test_condition_two' => [
            'id' => 'test_condition_two',
            'condition_met' => FALSE,
            'negate' => FALSE,
          ],
        ],
      ],
    ];
    $view->save();

    // Get the view executable again.
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');

    /** @var \Drupal\views\Plugin\views\access\AccessPluginBase $access_plugin */
    $access_plugin = $display->getPlugin('access');

    // Assert access is forbidden when one condition is not met.
    $this->assertTrue($access_plugin->access($this->user));
  }

  /**
   * Tests the summary title method.
   */
  public function testSummaryTitle(): void {
    $view = View::load('test_access_conditions');
    // Configure the view with our test condition.
    $display_options = $view->getDisplay('default');
    $display_options['display_options']['access'] = [
      'type' => 'views_access_conditions',
      'options' => [
        'conditions' => [
          'test_condition' => [
            'id' => 'test_condition',
            'condition_met' => TRUE,
            'negate' => FALSE,
          ],
        ],
      ],
    ];
    $view->save();

    // Get the view executable.
    $executable = $view->getExecutable();
    $executable->setDisplay('default');
    $display = $executable->displayHandlers->get('default');
    $access_plugin = $display->getPlugin('access');

    // Assert the summary title is correct.
    $this->assertEquals('Test condition: Test condition is met', $access_plugin->summaryTitle());
  }

}
